/*
 * Program testowy MPU-6050 dla Arduino, sprawdzony na module ESP32
 * wykorzysuje bibliotekę MPU6050_tockn v.1.5.2
 * wersja 1.2
 * aries-rs https://ars.info.pl/
 */
#include <MPU6050_tockn.h>
#include <Wire.h>

#include <Arduino.h>
#include <WiFi.h>
#include <AsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include <Arduino_JSON.h>
#include "SPIFFS.h"


MPU6050 mpu6050(Wire);

// Replace with your network credentials
const char* ssid = "REPLACE_WITH_YOUR_SSID";
const char* password = "REPLACE_WITH_YOUR_PASSWORD";

// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

// Create an Event Source on /events
AsyncEventSource events("/events");

// Json Variable to Hold Sensor Readings
JSONVar readings;

// Timer variables
unsigned long lastTime = 0;  
unsigned long lastTimeTemperature = 0;
unsigned long lastTimeAcc = 0;
unsigned long gyroDelay = 10;
unsigned long temperatureDelay = 1000;
unsigned long accelerometerDelay = 200;
//zmienne żyroskopu, akcelerometry i temperatury
float gyroX, gyroY, gyroZ;
float accX, accY, accZ;
float temperature;

bool reset_flag;
float gyroZ_reset_radian=0;
/*
 * procedura testu podłączenia MPU-6050 do magistrali I2C
 * 0x68 7 bitowy adres MPU-6050
 * w: true -MPU-6050 podłączony i wykryty
 */
bool testPodlaczenia_MPU6050(void)
{
#define MPU6050_I2CADDR_DEFAULT                                                \
  0x68                         ///< MPU6050 standardowy 7 bitowy adres i2c addres
#define MPU6050_DEVICE_ID 0x68 ///< prawidłowa wartość odpowiedzi na zapytanie MPU6050_WHO_AM_I 
#define MPU6050_WHO_AM_I 0x75  ///< rejestr ID zawierający bajt identyfikujący układ

  Wire.beginTransmission(MPU6050_I2CADDR_DEFAULT);
  Wire.write(MPU6050_WHO_AM_I);
  Wire.endTransmission();

  Wire.requestFrom(MPU6050_I2CADDR_DEFAULT, 1);    // request 1 byte from slave device #2

  while(Wire.available())    // slave may send less than requested
  {
    char c = Wire.read();    // receive a byte as character
    if (c ==MPU6050_DEVICE_ID) return true;
    else return false;
    //Serial.print(c);         // print the character
  }
  return false;
}
/*
 * inicjacja Systemu plików SPIFFS
 */
void initSPIFFS() {
  if (!SPIFFS.begin()) {
    Serial.println("An error has occurred while mounting SPIFFS");
  }
  Serial.println("SPIFFS mounted successfully");
}
/*
Initialize WiFi
*/
void initWiFi() {
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("");
  Serial.print("Connecting to WiFi...");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(1000);
  }
  Serial.println("");
  Serial.println(WiFi.localIP());
}
/*
 * zdarzenie odczytu żyroskopu
 */
String getGyroReadings(){
 
  mpu6050.update();
  gyroX =mpu6050.getAngleX()*0.01745329252;//zamiana stopnie na radiany
  gyroY =mpu6050.getAngleY()*0.01745329252;
  gyroZ =mpu6050.getAngleZ()*0.01745329252;

  gyroZ -=gyroZ_reset_radian;
  
  readings["gyroX"] = String(gyroX);
  readings["gyroY"] = String(gyroY);
  readings["gyroZ"] = String(gyroZ);

  String jsonString = JSON.stringify(readings);
  return jsonString;
}
/*
 * zdarzenie odczytu akcelerometru
 */
String getAccReadings() {

  mpu6050.update();
  accX =mpu6050.getAccX();//getAccX
  accY =mpu6050.getAccY();
  accZ =mpu6050.getAccZ();

  accX *=10;
  accY *=10;
  accZ *=10;
  
  readings["accX"] = String(accX);
  readings["accY"] = String(accY);
  readings["accZ"] = String(accZ);
  String accString = JSON.stringify (readings);
  return accString;
}
/*
 * zdarzenie odczytu temperatury
 */
 String getTemperature(){

  mpu6050.update();
  temperature = mpu6050.getTemp();
  return String(temperature);
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  Wire.begin();
  bool status =testPodlaczenia_MPU6050();
  if (status==true)
  {
    Serial.println("MPU6050 Found!");
  }
  else
  {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }    
  }
  initWiFi();
  initSPIFFS();   
  
  mpu6050.begin();
  mpu6050.calcGyroOffsets(true);
  reset_flag =false;
  
  // Handle Web Server
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/index.html", "text/html");
  });

  server.serveStatic("/", SPIFFS, "/");

  server.on("/reset", HTTP_GET, [](AsyncWebServerRequest *request){
    reset_flag =true;
    request->send(200, "text/plain", "OK");    
  });

  server.on("/resetX", HTTP_GET, [](AsyncWebServerRequest *request){
    gyroX=0;
    request->send(200, "text/plain", "OK");
  });

  server.on("/resetY", HTTP_GET, [](AsyncWebServerRequest *request){
    gyroY=0;
    request->send(200, "text/plain", "OK");
  });

  server.on("/resetZ", HTTP_GET, [](AsyncWebServerRequest *request){
    gyroZ=0;
    request->send(200, "text/plain", "OK");
  });

  // Handle Web Server Events
  events.onConnect([](AsyncEventSourceClient *client){
    if(client->lastId()){
      Serial.printf("Client reconnected! Last message ID that it got is: %u\n", client->lastId());
    }
    // send event with message "hello!", id current millis
    // and set reconnect delay to 1 second
    client->send("hello!", NULL, millis(), 10000);
  });
  server.addHandler(&events);

  server.begin();
}

void loop() {
  // put your main code here, to run repeatedly:
  if ((millis() - lastTime) > gyroDelay) {
    // Send Events to the Web Server with the Sensor Readings
    events.send(getGyroReadings().c_str(),"gyro_readings",millis());
    lastTime = millis();    
  }
  if ((millis() - lastTimeAcc) > accelerometerDelay) {
    // Send Events to the Web Server with the Sensor Readings
    events.send(getAccReadings().c_str(),"accelerometer_readings",millis());
    lastTimeAcc = millis();
  }
  if ((millis() - lastTimeTemperature) > temperatureDelay) {
    // Send Events to the Web Server with the Sensor Readings
    events.send(getTemperature().c_str(),"temperature_reading",millis());
    lastTimeTemperature = millis();
  }

  if (reset_flag ==true)
  {
    if ((millis() - lastTime) <2)
    {
    lastTime = millis();
    lastTimeAcc = millis();
    lastTimeTemperature = millis();
    
    gyroZ_reset_radian =gyroZ + gyroZ_reset_radian;
    reset_flag =false;  
    }
  }
}
